package com.cch_lab.android.ware.sample.mymessageapi;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.app.Fragment;
import android.content.Context;
import android.graphics.Color;
import android.os.Bundle;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

/**
 * Created by rie on 14/09/11.
 */
public class MyLayoutUtil {
    public static final String TAG = "LayoutUtil";

    private Context        mContext;

    public MyLayoutUtil(Context context) {
        this.mContext = context;
    }

    public void sample(final Context context) {
        String      title   = "test";
        String      message = "test message.";
        int         iconRes = 0;
        ButtonArray buttons = null;

        Log.d(TAG, "sample start");

        // View をつくる
        //View layout = inflater.inflate(R.layout.my_card_fragment, container, false);
        //layout.setBackgroundColor(Color.parseColor(color));

        //レイアウト・ユーティリティ
        MyLayoutUtil util = new MyLayoutUtil(context);

        // ルート View をつくる
        LinearLayout rootView = util.createLinearLayout();

        if (iconRes != 0) {
            ImageView iconView = util.createImageView(iconRes, true);
            rootView.addView(iconView, util.getLinearLayoutChildParams());
        }
        if (title != null) {
            TextView titleView = util.createTextView(title, true, 20, Color.BLACK);
            rootView.addView(titleView, util.getLinearLayoutChildParams());
        }
        if (message != null) {
            TextView messageView = util.createTextView(message, true, 12, Color.BLACK);
            rootView.addView(messageView, util.getLinearLayoutChildParams());
        }
        if (buttons != null && buttons.getButtonInfoArray() != null) {
            if (!buttons.isMultiListener()) {
                //複数ボタンの統一リスナ(統一遷移先)を設定する場合
                ButtonInfo[] buttonArray = buttons.getButtonInfoArray();
                LinearLayout wrapLayout  = null;
                Button       prevButton  = null;
                int          index       = 0;
                for (final ButtonInfo buttonInfo : buttonArray) {
                    Button button = util.createButton(buttonInfo.getLabel(), true, 12, Color.BLACK, Color.LTGRAY,
                            new View.OnClickListener(){
                                @Override
                                public void onClick(View v) {
                                    Toast.makeText(context, "Click" + buttonInfo.getLabel(), Toast.LENGTH_LONG).show();
                                }
                            });
                    int column = index % 3;
                    if (column == 0) {
                        wrapLayout = util.createWrapLayout();
                        rootView.addView(wrapLayout, util.getLinearLayoutChildParams());
                        rootView.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
                    }
                    wrapLayout.addView(button, util.getWrapLayoutChildParams(70, 70));

                    prevButton = button;
                    index++;
                }
            } else {
                //複数ボタンの個別リスナ(個別遷移先)を設定する場合
            }
        }

        //コンテナ View に追加
        //container.addView(rootView, container.getLayoutParams());

        util.analyzedLayoutStructure(rootView);
        Log.d(TAG, "sample end");
    }




    public LinearLayout createLinearLayout(){
        //View配置用レイアウト
        LinearLayout layout = new LinearLayout(this.mContext);
        Log.d(TAG, "createLinearLayout ID=" + layout.getId());
        //layout.setId(screenLayout.getId());
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.setLayoutParams(getLinearLayoutParams());
        return layout;
    }

    public LinearLayout createLinearLayout(int color, int backgroundImage){
        //View配置用レイアウト
        LinearLayout layout = new LinearLayout(this.mContext);
        Log.d(TAG, "createLinearLayout ID=" + layout.getId());
        //layout.setId(screenLayout.getId());
        layout.setOrientation(LinearLayout.VERTICAL);
        layout.setLayoutParams(getLinearLayoutParams());
        if (color != 0) {
            //color が ０でないなら背景色設定
            layout.setBackgroundColor(color);
        } else
        if (backgroundImage != 0) {
            //backgroundImage が 0 でないなら背景画像設定
            layout.setBackgroundResource(backgroundImage);
        }
        layout.setPadding(10,10,10,10);
        return layout;
    }

    public LinearLayout.LayoutParams getLinearLayoutParams(){
        LinearLayout.LayoutParams params =
                new LinearLayout.LayoutParams(
                        LinearLayout.LayoutParams.MATCH_PARENT,
                        LinearLayout.LayoutParams.MATCH_PARENT);
        return params;
    }

    public LinearLayout.LayoutParams getLinearLayoutChildParams(){
        LinearLayout.LayoutParams params;
        params = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.WRAP_CONTENT,
                LinearLayout.LayoutParams.WRAP_CONTENT);
        return params;
    }

    public LinearLayout createWrapLayout(){
        //View配置用レイアウト
        LinearLayout layout = new LinearLayout(this.mContext);
        Log.d(TAG, "createWrapLayout ID=" + layout.getId());
        //layout.setId(rootViewGroup.getId());
        layout.setLayoutParams(getWrapLayoutParams());
        return layout;
    }

    public LinearLayout.LayoutParams getWrapLayoutParams(){
        LinearLayout.LayoutParams params;
        params = new LinearLayout.LayoutParams(
                LinearLayout.LayoutParams.MATCH_PARENT,
                LinearLayout.LayoutParams.WRAP_CONTENT);
        return params;
    }

    public LinearLayout.LayoutParams getWrapLayoutChildParams(int viewWidth, int viewHeight){
        LinearLayout.LayoutParams params;
//        params = new LinearLayout.LayoutParams(
//                LinearLayout.LayoutParams.WRAP_CONTENT,
//                LinearLayout.LayoutParams.WRAP_CONTENT);
        params = new LinearLayout.LayoutParams(viewWidth,viewHeight);
        params.setMargins(2,2,2,2);
        return params;
    }

    public RelativeLayout createRelativeLayout(){
        //View配置用レイアウト
        RelativeLayout layout = new RelativeLayout(this.mContext);
        Log.d(TAG, "createRelativeLayout ID=" + layout.getId());
        //layout.setId(rootViewGroup.getId());
        layout.setLayoutParams(getRelativeLayoutParams());
        return layout;
    }

    public RelativeLayout.LayoutParams getRelativeLayoutParams(){
        RelativeLayout.LayoutParams params =
                new RelativeLayout.LayoutParams(
                        RelativeLayout.LayoutParams.MATCH_PARENT,
                        RelativeLayout.LayoutParams.MATCH_PARENT);
        return params;
    }

    public RelativeLayout.LayoutParams getRelativeLayoutChildParams(
            int width, int height,
            int align_horizontal, int align_vertical){

        RelativeLayout.LayoutParams params = new RelativeLayout.LayoutParams(width, height);
        params.addRule(align_horizontal);
        params.addRule(align_vertical);
        return params;
    }

    public ImageView createImageView(int drawableId, boolean isVisible){
        ImageView imageView = new ImageView(this.mContext);
        imageView.setBackgroundResource( drawableId );
        imageView.setVisibility( isVisible ? View.VISIBLE : View.INVISIBLE );
        return imageView;
    }

    public TextView createTextView(String message, boolean isVisible,
                                   int textSize, int textColor){
        TextView textView = new TextView(this.mContext);
        textView.setText( message );
        textView.setTextSize( textSize );
        textView.setTextColor( textColor );
        textView.setVisibility( isVisible ? View.VISIBLE : View.INVISIBLE );
        return textView;
    }

    public Button createButton(String label, boolean isVisible,
                               int textSize, int textColor,
                               int backColor, View.OnClickListener listener){
        Button button = new Button(this.mContext);
        button.setText(label);
        button.setTextSize(textSize);
        button.setTextColor(textColor);
//        button.setBackgroundColor(R.drawable.button_state);
//        button.setBackgroundColor(backColor);
        button.setOnClickListener(listener);
        button.setVisibility(isVisible ? View.VISIBLE : View.INVISIBLE);
        return button;
    }


    public static class ButtonInfo {
        private View.OnClickListener listener;
        private String               label;

        public ButtonInfo(View.OnClickListener listener, String label) {
            this.listener = listener;
            this.label    = label;
        }

        public View.OnClickListener getListener() {
            return listener;
        }
        public String getLabel() {
            return label;
        }
    }
    public static class ButtonArray {
        private boolean      isMultiListener;
        private ButtonInfo[] buttonInfoArray;

        /**
         * コンストラクタ.
         * <p>１つの代表リスナで複数ボタン(ラベル)を管理する場合のコンストラクタ<</p>
         * @param listener 代表Buttonクリック・リスナ
         * @param label    各Buttonラベル
         */
        public ButtonArray(View.OnClickListener listener, String ... label) {
            isMultiListener = false;
            String labels[] = label;
            if (listener != null && labels != null) {
                buttonInfoArray = new ButtonInfo[label.length];
                for (int index = 0; index < labels.length; index++) {
                    buttonInfoArray[index] = new ButtonInfo(listener, labels[index]);
                }
            }
        }

        /**
         * コンストラクタ.
         * <p>複数ボタンが個別にリスナとラベルを管理する場合のコンストラクタ<</p>
         * @param button 各Button (個別の遷移先とラベルを持つ)
         */
        public ButtonArray(ButtonInfo ... button) {
            isMultiListener = true;
            if (button != null) {
                buttonInfoArray = button;
            }
        }

        public boolean isMultiListener() {
            return isMultiListener;
        }

        public ButtonInfo[] getButtonInfoArray() {
            return buttonInfoArray;
        }
    }


    public void analyzedLayoutStructure(View rootView) {
        if (rootView == null) {
            Log.d(TAG, "analyzedLayoutStructure  root view=null");
            return;
        }

        //ルートビューの親Viewチェック
        View parentView = rootView.getRootView();
        if (parentView != null) {
            Log.d(TAG, "analyzedLayoutStructure  parent view=" + parentView.getClass().getSimpleName());
        } else {
            Log.d(TAG, "analyzedLayoutStructure  parent view=none");
        }

        //ルートビューのViewチェック
        Log.d(TAG, "analyzedLayoutStructure  root view=" + rootView.getClass().getSimpleName());

        //ルートビュー直下の子Viewチェック
        if (rootView instanceof ViewGroup) {
            int childLevel = 0;
            analyzedLayoutStructure(childLevel, (ViewGroup)rootView);
        }

    }
    private void analyzedLayoutStructure(int childLevel, ViewGroup rootView) {
        //ルートビュー直下の子Viewチェック
        int childCount = rootView.getChildCount();
        for (int index = 0; index < childCount; index++) {
            View childView = rootView.getChildAt(index);
            Log.d(TAG, "analyzedLayoutStructure  "
                    + getChildLevelSpan(childLevel)
                    + "child view[" + childLevel + "][" + index + "]=" + childView.getClass().getSimpleName());
            if (childView instanceof ViewGroup) {
                analyzedLayoutStructure(childLevel+1, (ViewGroup)childView);
            }
        }
    }
    private String getChildLevelSpan(int childLevel) {
        StringBuilder sb = new StringBuilder();
        for(int index = 0; index < childLevel; index++){
            sb.append("  ");
        }
        return sb.toString();
    }




    public static class MyFragment extends Fragment {
        private LinearLayout rootView;
        private ImageView    iconView;
        private TextView     titleView;
        private TextView     messageView;
        private Button[]     buttonArray;

        private String       title;
        private String       message;
        private int          iconRes;
        private ButtonArray  buttons;
        private int          backgroundColor;
        private int          backgroundImage;
        private int          gravity = Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL;

        private OnActivityCreatedListener onActivityCreated;

        public MyFragment(){
            super();
        }
        @SuppressLint("ValidFragment")
        public MyFragment(Bundle savedInstanceState,
                          String title,
                          String message,
                          int iconRes,
                          ButtonArray buttons,
                          int bgImage,
                          int bgColor,
                          int gravity){
            this();
            this.setArguments(savedInstanceState);
            this.title           = title;
            this.message         = message;
            this.iconRes         = iconRes;
            this.buttons         = buttons;
            this.backgroundImage = bgImage;
            this.backgroundColor = bgColor;
            this.gravity         = gravity;
        }

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
            Log.d(TAG, "onCreateView start");

            // View をつくる
            //View layout = inflater.inflate(R.layout.my_card_fragment, container, false);
            //layout.setBackgroundColor(Color.parseColor(color));

            //レイアウト・ユーティリティ
            MyLayoutUtil util = new MyLayoutUtil(getActivity());

            // ルート View をつくる
            rootView = null;
            if (this.backgroundImage != 0) {
                rootView = util.createLinearLayout(0, this.backgroundImage);
            } else
            if (this.backgroundColor != 0) {
                rootView = util.createLinearLayout(this.backgroundColor, 0);
            } else {
                rootView = util.createLinearLayout(Color.WHITE, 0);
            }

            if (this.iconRes != 0) {
                iconView = util.createImageView(this.iconRes, true);
                rootView.addView(iconView, util.getLinearLayoutChildParams());
            }
            if (this.title != null) {
                titleView = util.createTextView(this.title, true, 20, Color.BLACK);
                rootView.addView(titleView, util.getLinearLayoutChildParams());
            }
            if (this.message != null) {
                messageView = util.createTextView(this.message, true, 12, Color.BLACK);
                rootView.addView(messageView, util.getLinearLayoutChildParams());
            }
            if (this.buttons != null && this.buttons.getButtonInfoArray() != null) {
                if (!buttons.isMultiListener()) {
                    //複数ボタンの統一リスナ(統一遷移先)を設定する場合
                    ButtonInfo[] buttonInfoArray = this.buttons.getButtonInfoArray();
                    LinearLayout wrapLayout = null;
                    Button prevButton = null;
                    int index = 0;
                    buttonArray = new Button[buttonInfoArray.length];
                    for (final ButtonInfo buttonInfo : buttonInfoArray) {
                        buttonArray[index] = util.createButton(
                                buttonInfo.getLabel(),
                                true,
                                12,
                                Color.BLACK,
                                Color.LTGRAY,
                                buttonInfo.getListener());
                        int column = index % 3;
                        if (column == 0) {
                            wrapLayout = util.createWrapLayout();
                            rootView.addView(wrapLayout, util.getLinearLayoutChildParams());
                            rootView.setGravity(Gravity.CENTER_HORIZONTAL | Gravity.CENTER_VERTICAL);
                        }
                        wrapLayout.addView(buttonArray[index],
                                util.getWrapLayoutChildParams(
                                    LinearLayout.LayoutParams.WRAP_CONTENT,
                                    LinearLayout.LayoutParams.WRAP_CONTENT));
                        prevButton = buttonArray[index];
                        index++;
                    }
                } else {
                    //複数ボタンの個別リスナ(個別遷移先)を設定する場合
                }
            }

            //コンテナ View に追加
            //container.addView(rootView, container.getLayoutParams());

            util.analyzedLayoutStructure(rootView);
            Log.d(TAG, "onCreateView end");
            return rootView;
        }

        @Override
        public void onActivityCreated(Bundle savedInstanceState) {
            super.onActivityCreated(savedInstanceState);
            if (onActivityCreated != null) {
                onActivityCreated.onActivityCreated(savedInstanceState);
            }
        }
        public void setOnActivityCreatedListener(OnActivityCreatedListener listener) {
            onActivityCreated = listener;
        }
        public static interface OnActivityCreatedListener {
            public void onActivityCreated(Bundle savedInstanceState);
        }

        public LinearLayout getRootView()    { return rootView;}
        public ImageView    getIconView()    { return iconView; }
        public TextView     getTitleView()   { return titleView; }
        public TextView     getMessageView() { return messageView; }
        public Button[]     getButtons()     { return buttonArray; }
   }

}
