package com.cch_lab.android.ware.sample.mymessageapi;


import android.util.Log;

import com.google.android.gms.wearable.DataEventBuffer;
import com.google.android.gms.wearable.MessageEvent;
import com.google.android.gms.wearable.WearableListenerService;

/**
 * MessageApi を利用した、メッセージ受信サービス．
 * <p>
 * MessageApi を利用した、スマートフォン → Wear
 * あるいは Wear → スマートフォン のメッセージ受信(リスナ)サービス・クラスです。
 * <ul>
 * <li>
 * AndroidManifest.xml ファイルの <application> 要素内には、以下のメタ-データ定義タグを追加してください。<br/>
 *  ＜meta-data<br/>
 *  　android:name="com.google.android.gms.version"<br/>
 *  　android:value="@integerogle_play_services_version" /＞<br/>
 * </li>
 * <li>
 * AndroidManifest.xml ファイルの <application> 要素内には、以下のサービス定義を追加してください。<br/>
 * ＜service android:name=".MyWearableListenerService"＞<br/>
 * 　＜intent-filter＞<br/>
 * 　＜action android:name="com.google.android.gms.wearable.BIND_LISTENER" /＞<br/>
 * 　＜/intent-filter＞<br/>
 * ＜/service＞<br/>
 * </li>
 * </ul>
 * </p>
 */
public class MyWearableListenerService extends WearableListenerService {
    private static final String TAG = "MyWearableListenerService";

    /**
     * メッセージ受信時対応メソッド．
     * @param messageEvent メッセージ・イベント
     */
    @Override
    public void onMessageReceived(MessageEvent messageEvent) {
        Log.d(TAG, "onMessageReceived  start");
        MyWearableListenerAction listenerAction = MyMessageApiUtil.getListenerAction();
        String receivePathQuery = null;
        if (listenerAction == null) {
            Log.d(TAG, "onMessageReceived  end(null)");
            return;
        } else {
            for (String targetQuery : MyMessageApiUtil.getsReceivePathQueryArray()) {

                if (MyMessageApiUtil.createMessagePath(targetQuery).equals(messageEvent.getPath())){
                    receivePathQuery = targetQuery;
                    break;
                }
            }
            if (receivePathQuery == null) {
                Log.d(TAG, "onMessageReceived  end(null)");
                return;
            }
        }
        listenerAction.onMessageReceived(receivePathQuery, messageEvent);
        Log.d(TAG, "onMessageReceived  end");
    }

    /**
     * データ受信時対応メソッド．
     * @param dataEvents データ・イベント(複数可)
     */
    @Override
    public void onDataChanged(DataEventBuffer dataEvents) {
        //データ受信には、対応していません。
    }

    /**
     * MessageApi を利用した、メッセージ受信アクション・インターフェース．
     * <p>
     * MessageApi を利用した、スマートフォン → Wear
     * あるいは Wear → スマートフォン のメッセージ受信アクション・インターフェースです。
     * </p>
     */
    public static interface MyWearableListenerAction {
        /**
         * メッセージ受信時対応メソッド．
         * @param messagePathQuery メッセージ識別用パス・クエリ部
         * @param messageEvent     メッセージ・イベント
         */
        public void onMessageReceived(String messagePathQuery, MessageEvent messageEvent);

        ///**
        // * データ受信時対応メソッド．
        // * @param dataEvents データ・イベント(複数可)
        // */
        //public void onDataChanged(DataEventBuffer dataEvents);
    }
}
