package com.cch_lab.android.ware.sample.mymessageapi;

import android.app.Activity;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Bundle;
import android.support.wearable.view.WatchViewStub;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.google.android.gms.wearable.MessageEvent;

public class MainActivity extends Activity implements View.OnClickListener {
    public static final String TAG = "MainActivity";

    private LinearLayout mRootLayout;

    @Override
    protected void onCreate(final Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_activity);
        final WatchViewStub stub = (WatchViewStub) findViewById(R.id.watch_view_stub);
        stub.setOnLayoutInflatedListener(new WatchViewStub.OnLayoutInflatedListener() {
            @Override
            public void onLayoutInflated(WatchViewStub stub) {
                mRootLayout = (LinearLayout) stub.findViewById(R.id.layout);
                final MyLayoutUtil.MyFragment wearFragment = new MyLayoutUtil.MyFragment(
                        savedInstanceState,
                        "MessageApi Sample",                  //titleC
                        "mobile にメッセージを送信します。",    //message
                        R.drawable.ic_launcher,               //iconRes;
                        new MyLayoutUtil.ButtonArray(
                                MainActivity.this,
                                "normal", "danger"),          //buttons
                        0,                                    //bgImage
                        Color.GREEN,                          //bgImage
                        Gravity.CENTER                        //gravity;
//                      Gravity.TOP | Gravity.LEFT            //gravity;
                );
                getFragmentManager().beginTransaction()
                        .add(R.id.layout, wearFragment)
                        .commit();
                wearFragment.setOnActivityCreatedListener(new MyLayoutUtil.MyFragment.OnActivityCreatedListener() {
                    @Override
                    public void onActivityCreated(Bundle savedInstanceState) {
                        //ボタンのリスナとして、当該ActivityのonClickメソッドをバインド
                        wearFragment.getButtons()[0].setOnClickListener(MainActivity.this);
                    }
                });
            }
        });

        //メッセージAPIユーティリティ初期化
        MyMessageApiUtil.init(
                this,
                new String[]{"mobile/normal", "mobile/danger", "wear/normal", "wear/danger"},
                new MyWearableListenerService.MyWearableListenerAction() {
                    public void onMessageReceived(final String messagePathQuery, final MessageEvent messageEvent) {
                        //Wearからのメッセージ受信時処理
                        Log.d(TAG, "message=" + new String(messageEvent.getData()) + "\n" + "query=" + messagePathQuery);
                        Toast.makeText(
                                MainActivity.this,
                                ("message=" + new String(messageEvent.getData()) + "\n"
                                        + "query=" + messagePathQuery),
                                Toast.LENGTH_LONG).show();
                    }
                });

        Log.d(TAG, "MyMessageApiUtil.init() executed");
    }

    @Override
    public void onClick(View v) {
        //mobileにメッセージ送信
        Button button = (Button)v;
        if (button.getText().equals("normal")) {
            MyMessageApiUtil.sendMessage("wear/normal", "message from wear at normal.");
        } else
        if (button.getText().equals("danger")) {
            MyMessageApiUtil.sendMessage("wear/danger", "message from wear at danger.");
        }
        Log.d(TAG, "wear -> onClick  sendMessage()");
    }
}
